package org.xiss.index.impl.persistence;

import java.io.IOException;
import java.util.Iterator;

import net.sf.xanswer.storage.map.PersistentException;

import org.dbxml.core.DBException;
import org.dbxml.core.data.Value;
import org.dbxml.core.filer.BTreeException;
import org.exist.storage.store.BFile;
import org.xiss.map.IBTree;
import org.xiss.map.KeyValuePair;

public class GenericBTree<K, V> implements IBTree<K, V> {
	private BFile myBFile;

	private ValueConverter myBridge;

	private BTreeCache<K, V> myCache;

	public static final int DEFAULT_CACHE_CAPACITY = 16;

	public GenericBTree(BFile file) {
		this(file, DEFAULT_CACHE_CAPACITY);
	}

	public GenericBTree(BFile file, int cacheCapacity) {
		myBFile = file;
		myCache = new BTreeCache<K, V>(this, cacheCapacity);

		myBridge = new ValueConverter();
	}

	public boolean containsKey(K key) throws PersistentException {
		return myCache.get(key) != null;
	}

	public V get(K key) throws PersistentException {
		return myCache.get(key);
	}

	V getNoCache(K key) throws PersistentException {
		Value value = myBFile.get(myBridge.serialize(key));
		if (value == null) {
			return null;
		}
		return (V) myBridge.deserialize(value);
	}

	public void put(K key, V value) throws PersistentException {
		myCache.put(key, value);
	}

	void putNoCache(K key, V value) throws PersistentException {
		myBFile.put(myBridge.serialize(key), myBridge.serialize(value)
				.getData(), true);
	}

	public void clear() {
		throw new UnsupportedOperationException(
				"Whoops, seems like there's no way to delete a "
						+ "PersistentMap");
	}

	public Iterator<KeyValuePair<K, V>> iterator() {
		try {
			flush();
			return new IteratorAdapter();
		} catch (PersistentException e) {
			throw new RuntimeException(e);
		}
	}

	public void flush() throws PersistentException {
		myCache.flush();
	}

	public void close() throws PersistentException {
		try {
			flush();
			/*
			 * System.out.println(myBFile.getFile().getAbsolutePath());
			 * System.out.println("Cache hits: "+myCache.getHits());
			 * System.out.println("Cache misses: "+myCache.getMisses());
			 * System.out.println("Gets from disk: "+myCache.getActualGets());
			 * System.out.println("Puts to disk: "+myCache.getActualPuts());
			 */
			myBFile.close();
		} catch (DBException e) {
			throw new PersistentException(e);
		}
	}

	public void open() throws PersistentException {
		try {
			myBFile.open();
		} catch (DBException e) {
			throw new PersistentException(e);
		}
	}

	public void drop() throws PersistentException {
		try {
			myBFile.drop();
		} catch (DBException e) {
			throw new PersistentException(e);
		}
	}

	private final class IteratorAdapter implements Iterator<KeyValuePair<K, V>> {
		private Iterator i;

		public IteratorAdapter() throws PersistentException {
			try {
				i = myBFile.getEntries().iterator();
			} catch (IOException e) {
				throw new PersistentException(e);
			} catch (BTreeException e) {
				throw new PersistentException(e);
			}
		}

		public boolean hasNext() {
			return i.hasNext();
		}

		public KeyValuePair<K, V> next() {
			Value[] values = (Value[]) i.next();
			try {
				return new KeyValuePair<K, V>((K) myBridge
						.deserialize(values[0]), (V) myBridge
						.deserialize(values[1]));
			} catch (PersistentException e) {
				throw new RuntimeException(e);
			}
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}
	}
}
