package org.xiss.index.impl.persistence;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.nio.ByteBuffer;

import net.sf.xanswer.storage.map.PersistentException;

import org.dbxml.core.data.Value;

class ValueConverter {
	private static final byte INTEGER_PREFIX = 0;

	private static final byte STRING_PREFIX = 1;

	private static final byte OBJECT_PREFIX = 2;

	Value serialize(Object x) throws PersistentException {
		Class<?> c = x.getClass();
		if (c == Integer.class) {
			ByteBuffer b = ByteBuffer.allocate(5);
			b.put(ValueConverter.INTEGER_PREFIX);
			b.putInt((Integer) x);
			return new Value(b.array());
		} else if (c == String.class) {
			byte[] bytes = ((String) x).getBytes();
			byte[] res = new byte[1 + bytes.length];
			res[0] = ValueConverter.STRING_PREFIX;
			System.arraycopy(bytes, 0, res, 1, bytes.length);
			return new Value(res);
		} else {
			try {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				bos.write(ValueConverter.OBJECT_PREFIX);
				ObjectOutputStream os = new ObjectOutputStream(bos);
				os.writeObject(x);
				return new Value(bos.toByteArray());
			} catch (IOException e) {
				throw new PersistentException(e);
			}
		}
	}

	Object deserialize(Value value) throws PersistentException {
		byte[] bytes = value.getData();
		if (bytes.length < 1) {
			throw new PersistentException(new RuntimeException(
					"Unexpected contents for deserialization"));
		}
		switch (bytes[0]) {
		case INTEGER_PREFIX:
			ByteBuffer buf = ByteBuffer.wrap(bytes);
			return buf.getInt(1);
		case STRING_PREFIX:
			return new String(bytes, 1, bytes.length - 1);
		case OBJECT_PREFIX:
			try {
				ByteArrayInputStream bis = new ByteArrayInputStream(bytes, 1,
						bytes.length - 1);
				ObjectInputStream ois = new ObjectInputStream(bis);
				return ois.readObject();
			} catch (Exception e) {
				throw new PersistentException(e);
			}
		default:
			throw new PersistentException(new RuntimeException(
					"Unknown prefix during deserialization"));
		}
	}
}
