package org.xiss.query.impl;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import org.xiss.index.IXissIndex;
import org.xiss.query.Chunk;
import org.xiss.query.IResult;
import org.xiss.query.IResultNode;

public class ChunkList extends AbstractResult {
	private List<Chunk> myChunks;

	private boolean myIsRootSorted;

	private boolean myIsDeepSorted;

	public ChunkList() {
		myChunks = new LinkedList<Chunk>();
	}

	public ChunkList(List<Chunk> chunks) {
		myChunks = chunks;
	}

	public List<Chunk> getChunks() {
		return myChunks;
	}

	public void addChunk(Chunk node) {
		myChunks.add(node);
	}

	public boolean isEmpty() {
		return myChunks.isEmpty();
	}

	public Iterable<IResultNode> getNodes(IXissIndex index) {
		return new Chunks2NodesAdapter(index, this);
	}

	public boolean isSortedByStart() {
		return myIsRootSorted;
	}

	public boolean isSortedByEnd() {
		return myIsDeepSorted;
	}

	public void setUnsorted() {
		myIsRootSorted = myIsDeepSorted = false;
	}

	public void setRootSorted() {
		myIsRootSorted = true;
	}

	public void setDeepSorted() {
		myIsDeepSorted = true;
	}

	public void sortByStart() {
		sort(true);
	}

	public void sortByEnd() {
		sort(false);
	}

	private void sort(final boolean sortByStartNotEnd) {
		Collections.sort(myChunks, new Comparator<Chunk>() {
			public int compare(Chunk a, Chunk b) {
				return Chunk.compare(a, b, sortByStartNotEnd);
			}
		});
	}

	public String toString() {
		return myChunks.toString();
	}

	public IResult copy() {
		List<Chunk> chunksCopy = new LinkedList<Chunk>();
		for (Chunk c : myChunks) {
			chunksCopy.add(new Chunk(c.Start, c.End));
		}

		return new ChunkList(chunksCopy);
	}
}
