package org.xiss.query.operations;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Stack;

import net.sf.xanswer.indexing.IndexException;

import org.xiss.index.ElementIndexEntry;
import org.xiss.index.IXissIndex;
import org.xiss.query.Chunk;
import org.xiss.query.IResult;
import org.xiss.query.IXissOperation;
import org.xiss.query.impl.ChunkList;

/**
 * Selects elements by presence and/or value of an attribute using EA-join on an
 * element set and an attribute set
 */
public class AttributeJoinOperation implements IXissOperation {
	private String myAttributeName;

	private String myAttributeValue;

	public AttributeJoinOperation(String name, String value) {
		myAttributeName = name;
		myAttributeValue = value;
	}

	public void run(IXissIndex index, Stack<IResult> context)
			throws IndexException {
		List<? extends ElementIndexEntry> ats = index
				.getAttributeIndexEntriesByNid(index
						.getNidByName(myAttributeName));
		IResult top = context.pop();

		List<Chunk> chunks = new LinkedList<Chunk>();
		ChunkList res = new ChunkList(chunks);

		// EA-join 'top' with 'ats'
		top.sortByEnd();
		res.setDeepSorted();

		Iterator<Chunk> ci = top.getChunks().listIterator();
		ListIterator<? extends ElementIndexEntry> ai = ats.listIterator();
		if (!ai.hasNext() || !ci.hasNext()) {
			context.push(res);
			return;
		}
		ElementIndexEntry atr = ai.next();
		Chunk chunk = ci.next();
		while (true) {
			if (atr.ParentId == chunk.End.Id) {
				if (isValueMatching(index, atr, myAttributeValue)) {
					// The resulting chunks must be just elements,
					// not element/@atr.
					res.addChunk(new Chunk(chunk.End, chunk.End));
				}
			}

			if (atr.Id > chunk.End.Id) {
				if (!ci.hasNext()) {
					break;
				}
				chunk = ci.next();
			} else {
				if (!ai.hasNext()) {
					break;
				}
				atr = ai.next();
			}
		}

		context.push(res);
	}

	private boolean isValueMatching(IXissIndex index, ElementIndexEntry atr,
			String value) throws IndexException {
		if (value == null) {
			return true;
		}
		int vid = atr.ValueId;
		return index.getValueByVid(vid).equals(value);
	}

	public String toString() {
		if (myAttributeValue != null) {
			return "AJOIN(" + myAttributeName + "=" + myAttributeValue + ")";
		}
		return "AJOIN(" + myAttributeName + ")";
	}
}
