package org.xiss.query.operations;

import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Stack;

import org.xiss.index.IXissIndex;
import org.xiss.query.Chunk;
import org.xiss.query.IResult;
import org.xiss.query.IXissOperation;
import org.xiss.query.impl.ChunkList;

/**
 * EE-joins two element sets
 */
public class JoinOperation implements IXissOperation {
	private int myDepth;

	private boolean myIsExactNotLowerBound;

	public JoinOperation(int depth, boolean isExactNotLowerBound) {
		myDepth = depth;
		myIsExactNotLowerBound = isExactNotLowerBound;
	}

	public void run(IXissIndex index, Stack<IResult> context) {
		IResult right = context.pop(), left = context.pop();
		if (!left.isSortedByStart()) {
			left.sortByStart();
		}
		if (!right.isSortedByStart()) {
			right.sortByStart();
		}

		List<Chunk> leftChunks = left.getChunks();
		List<Chunk> rightChunks = right.getChunks();

		List<Chunk> candidates = new LinkedList<Chunk>();

		ChunkList res = new ChunkList();
		res.setRootSorted();

		ListIterator<Chunk> leftIterator = leftChunks.listIterator();

		Chunk newCandidate = null;
		for (Chunk rightChunk : rightChunks) {
			while (leftIterator.hasNext()) {
				if (newCandidate == null)
					newCandidate = leftIterator.next();
				if (newCandidate.Start.Id > rightChunk.End.getNextId()) {
					break;
				} else if (newCandidate.Start.getNextId() < rightChunk.End.Id) {
					newCandidate = leftIterator.next();
					continue;
				} else {
					candidates.add(newCandidate);
					newCandidate = null;
				}
			}
			if (!leftIterator.hasNext()) {
				if (newCandidate != null) {
					candidates.add(newCandidate);
					newCandidate = null;
				}
			}

			ListIterator<Chunk> candidateIterator = candidates.listIterator();
			Chunk candidate = null;
			while (candidateIterator.hasNext()
					&& (candidate = candidateIterator.next()).Start.getNextId() < rightChunk.End.Id) {
				candidateIterator.remove();
			}

			while (candidate != null) {
				if (candidate.Start.Id < rightChunk.Start.Id
						&& candidate.Start.getNextId() >= rightChunk.End
								.getNextId()
						&& depthTest(candidate, rightChunk)) {
					res.addChunk(new Chunk(candidate.Start, rightChunk.End));
				} else if (candidate.Start.getNextId() < rightChunk.Start.Id) {
					break;
				}
				if (candidateIterator.hasNext()) {
					candidate = candidateIterator.next();
				} else {
					break;
				}
			}

		}

		/*
		 * ListIterator<Chunk> startPoint = rightChunks.listIterator();
		 * 
		 * cycleOnLeft: for (Chunk top : leftChunks) { ListIterator<Chunk>
		 * rightIter = startPoint.clone(); boolean found = false; while
		 * (rightIter.hasNext()) { Chunk bottom = rightIter.next(); if
		 * (bottom.Start.getId() > top.End.getId() + top.End.getSize()) continue
		 * cycleOnLeft; if ((bottom.Start.getId() > top.End.getId()) &&
		 * depthTest(top, bottom)) { res.addChunk(new Chunk(top.Start,
		 * bottom.End)); found = true; } else { if (found) break; startPoint =
		 * rightIter.clone(); } } }
		 */

		context.push(res);
	}

	private boolean depthTest(Chunk top, Chunk bottom) {
		return ((myIsExactNotLowerBound) ? (bottom.Start.Depth == top.End.Depth
				+ myDepth) : (bottom.Start.Depth >= top.End.Depth + myDepth));
	}

	public String toString() {
		if (myIsExactNotLowerBound) {
			return "JOIN(" + Integer.valueOf(myDepth).toString() + ")";
		} else {
			return "JOIN(" + Integer.valueOf(myDepth).toString() + "..INF)";
		}
	}
}
