package org.xiss.index.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;

import junit.framework.Assert;
import junit.framework.TestCase;
import net.sf.xanswer.indexing.IndexException;

import org.xiss.index.XissIndexFactory;

public class IndexTest extends TestCase {
	public void setUp() {

	}

	public void testUpdateNameTable() {
		XissIndexFactory factory = XissIndexFactory.newInstance();
		XissIndexImpl index = null;
		try {
			uglyHackRemoveDatabase("test");
			index = (XissIndexImpl) factory.createEmpty("test");
			int nid;

			nid = index.updateNameTable("element");
			Assert.assertEquals(nid, 0);
			nid = index.updateNameTable("second");
			Assert.assertEquals(nid, 1);
			nid = index.updateNameTable("johnny");
			Assert.assertEquals(nid, 2);
			nid = index.updateNameTable("element");
			Assert.assertEquals(nid, 0);
			nid = index.updateNameTable("second");
			Assert.assertEquals(nid, 1);

			final int iterations = 100;
			ArrayList<String> names = new ArrayList<String>(iterations);
			for (int i = 0; i < iterations; ++i) {
				names.add("" + i);
			}
			Collections.shuffle(names);
			for (int i = 0; i < iterations; ++i) {
				Assert.assertEquals(index.updateNameTable("" + names.get(i)),
						i + 3);
			}

			for (int i = 0; i < iterations; ++i) {
				Assert.assertEquals(index.updateNameTable("" + names.get(i)),
						i + 3);
			}

		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} finally {
			try {
				index.close();
			} catch (IndexException e) {
			}
		}
	}

	public void testUpdateValueTable() {
		// Exactly the same
		XissIndexFactory factory = XissIndexFactory.newInstance();
		XissIndexImpl index = null;
		try {
			uglyHackRemoveDatabase("test");
			index = (XissIndexImpl) factory.createEmpty("test");
			int nid;

			int distinct = 0;
			nid = index.updateValueTable("element");
			Assert.assertEquals(nid, 0);
			nid = index.updateValueTable("second");
			Assert.assertEquals(nid, 1);
			nid = index.updateValueTable("johnny");
			Assert.assertEquals(nid, 2);
			nid = index.updateValueTable("element");
			Assert.assertEquals(nid, 0);
			nid = index.updateValueTable("second");
			Assert.assertEquals(nid, 1);

			distinct = 3;

			final int iterations = 100;
			ArrayList<String> names = new ArrayList<String>(iterations);
			for (int i = 0; i < iterations; ++i) {
				names.add("" + i);
			}
			Collections.shuffle(names);
			for (int i = 0; i < iterations; ++i) {
				String value = names.get(i);
				int vid = index.updateValueTable(value);
				Assert.assertEquals(vid, (i + distinct));
			}

			for (int i = 0; i < iterations; ++i) {
				String value = names.get(i);
				int vid = index.updateValueTable(value);
				Assert.assertEquals(vid, (i + distinct));
			}
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} finally {
			try {
				index.close();
			} catch (IndexException e) {
			}
		}
	}

	public void testGetNodeNameAndValue() {
		XissIndexFactory factory = XissIndexFactory.newInstance();
		XissIndexImpl index = null;
		try {
			uglyHackRemoveDatabase("test");
			index = (XissIndexImpl) factory.createEmpty("test");
			int nid = index.updateNameTable("element");
			int vid = index.updateNameTable("value");
			StructureIndexEntryImpl entry = new StructureIndexEntryImpl(0, 1,
					nid, vid, 0, 0, 0, 0);
			index.addStructureIndexEntry(entry);

			Assert.assertEquals(index.getNodeName(0), "element");
			Assert.assertEquals(index.getNodeValue(0), "value");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} finally {
			try {
				index.close();
			} catch (IndexException e) {
			}
		}
	}

	private void uglyHackRemoveDatabase(String name) {
		File f = new File(name);
		deleteRecursive(f);
	}

	private void deleteRecursive(File f) {
		if (!f.exists()) {
			return;
		}
		if (f.isDirectory()) {
			for (File content : f.listFiles()) {
				deleteRecursive(content);
			}
		}
		f.delete();
	}
}
